@@ -25,10 +25,30 @@ mixed_model <- function (fixed, random, data, family, na.action = na.exclude,
     data <- as.data.frame(data) # in case 'data' is a tibble
     mfX <- model.frame(terms(fixed, data = data), data = data, na.action = na.action)
     na_exclude <- attr(mfX, "na.action")
+    form_random <- getRE_Formula(random)
+    mfZ <- model.frame(terms(form_random, data = data), data = data)
+    na_exclude_z <- attr(mfZ, "na.action")
     if (!is.null(zi_fixed)) { 
         mfX_zi <- model.frame(terms(zi_fixed, data = data), data = data, 
                               na.action = na.action)
-        na_exclude <- union(attr(mfX_zi, "na.action"), na_exclude)
+        na_exclude_zi <- attr(mfX_zi, "na.action")
+    } else {
+        na_exclude_zi <- NULL
+    }
+    ind <- !unique(names(c(na_exclude_z, na_exclude_zi))) %in% names(na_exclude)
+    if (length(ind) && any(ind)) {
+        keep <- !row.names(mfX) %in% unique(names(c(na_exclude_z, na_exclude_zi)))[ind]
+        mfX <- mfX[keep, , drop = FALSE]
+    }
+    ind <- !unique(names(c(na_exclude, na_exclude_zi))) %in% names(na_exclude_z)
+    if (length(ind) && any(ind)) {
+        keep <- !row.names(mfZ) %in% unique(names(c(na_exclude, na_exclude_zi)))[ind]
+        mfZ <- mfZ[keep, , drop = FALSE]
+    }
+    ind <- !unique(names(c(na_exclude, na_exclude_z))) %in% names(na_exclude_zi)
+    if (length(ind) && any(ind) && !is.null(zi_fixed)) {
+        keep <- !row.names(mfX_zi) %in% unique(names(c(na_exclude, na_exclude_z)))[ind]
+        mfX_zi <- mfX_zi[keep, , drop = FALSE]
     }
     termsX <- terms(mfX)
     y <- model.response(mfX)
@@ -40,16 +60,12 @@ mixed_model <- function (fixed, random, data, family, na.action = na.exclude,
     }
     X <- model.matrix(termsX, mfX)
     offset <- model.offset(mfX)
-    form_random <- getRE_Formula(random)
-    mfZ <- model.frame(terms(form_random, data = data), data = data)
-    if (!is.null(na_exclude))
-        mfZ <- mfZ[-na_exclude, , drop = FALSE]
     termsZ <- terms(mfZ)
     Z <- model.matrix(termsZ, mfZ)
     id_nam <- all.vars(getID_Formula(random))
     id_orig <- model.frame(terms(getID_Formula(random)), data)[[1L]]
     if (!is.null(na_exclude))
-        id_orig <- id_orig[-na_exclude]
+        id_orig <- id_orig[-unique(c(na_exclude, na_exclude_z, na_exclude_zi))]
     id <- match(id_orig, unique(id_orig))
     ###########################
     # Zero inflation part
@@ -60,8 +76,6 @@ mixed_model <- function (fixed, random, data, family, na.action = na.exclude,
              "'zi_fixed' needs to be defined, and potentially also argument 'zi_random'.")
     }
     if (!is.null(zi_fixed)) {
-        if (!is.null(na_exclude)) 
-            mfX_zi <- mfX_zi[-na_exclude, ]
         termsX_zi <- terms(mfX_zi)
         X_zi <- model.matrix(termsX_zi, mfX_zi)
         offset_zi <- model.offset(mfX_zi)
@@ -87,11 +101,11 @@ mixed_model <- function (fixed, random, data, family, na.action = na.exclude,
     ###########################
     # control settings
     con <- list(iter_EM = 30, iter_qN_outer = 15, iter_qN = 10, iter_qN_incr = 10,
-                optim_method = "BFGS", parscale_betas = 0.1, parscale_D = 0.01,
-                parscale_phis = 0.01, parscale_gammas = 0.01, tol1 = 1e-03, tol2 = 1e-04, 
-                tol3 = 1e-07, numeric_deriv = "fd", nAGQ = if (nRE < 3) 11 else 7, 
-                update_GH_every = 10, max_coef_value = 10, max_phis_value = exp(10), 
-                verbose = FALSE, optimParallel = FALSE)
+                optimizer = "optim", optim_method = "BFGS", parscale_betas = 0.1, 
+                parscale_D = 0.01, parscale_phis = 0.01, parscale_gammas = 0.01, 
+                tol1 = 1e-04, tol2 = 1e-05, tol3 = 1e-08, numeric_deriv = "fd", 
+                nAGQ = if (nRE < 3) 11 else 7, update_GH_every = 10, max_coef_value = 12, 
+                max_phis_value = exp(10), verbose = FALSE)
     control <- c(control, list(...))
     namC <- names(con)
     con[(namc <- names(control))] <- control
@@ -119,7 +133,8 @@ mixed_model <- function (fixed, random, data, family, na.action = na.exclude,
                    list(gammas = glm.fit(X_zi, as.numeric(y == 0), 
                                          family = binomial())$coefficients))
         if (family$family %in% c("zero-inflated poisson", "zero-inflated negative binomial",
-                                 "hurdle poisson", "hurdle negative binomial"))
+                                 "hurdle poisson", "hurdle negative binomial", 
+                                 "Conway Maxwell Poisson"))
             inits$betas <- glm.fit(X, y, family = poisson())$coefficients
     }
     ##########################
@@ -179,7 +194,7 @@ mixed_model <- function (fixed, random, data, family, na.action = na.exclude,
     if (has_phis) {
         if (family$family %in% c("negative binomial", "zero-inflated negative binomial",
                                  "hurdle negative binomial", "hurdle log-normal",
-                                 "beta", "hurdle beta")) {
+                                 "beta", "hurdle beta", "Conway Maxwell Poisson")) {
             n_phis <- 1
         } else if (is.null(n_phis)) {
             stop("argument 'n_phis' needs to be specified.\n")
