@@ -53,6 +53,11 @@ extract <- function (obj) {
 
 sapply(models, extract)
 
+## ------------------------------------------------------------------------
+km <- glm(y ~ sex * time, data = DF, family = binomial())
+
+anova(fm1, km)
+
 ## ------------------------------------------------------------------------
 fm2 <- mixed_model(fixed = y ~ sex * time, random = ~ time || id, data = DF,
                    family = binomial())
@@ -60,12 +65,12 @@ fm2 <- mixed_model(fixed = y ~ sex * time, random = ~ time || id, data = DF,
 ## ------------------------------------------------------------------------
 anova(fm1, fm2)
 
-## ------------------------------------------------------------------------
-fm3 <- mixed_model(fixed = y ~ sex * time, random = ~ time | id, data = DF,
-                   family = binomial())
+## ---- eval = FALSE-------------------------------------------------------
+#  fm3 <- mixed_model(fixed = y ~ sex * time, random = ~ time | id, data = DF,
+#                     family = binomial())
 
-## ------------------------------------------------------------------------
-anova(fm2, fm3)
+## ---- eval = FALSE-------------------------------------------------------
+#  anova(fm2, fm3)
 
 ## ------------------------------------------------------------------------
 set.seed(1234)
@@ -92,19 +97,59 @@ eta_y <- drop(X %*% betas + b[DF$id])
 # we simulate Poisson longitudinal data
 DF$y <- rpois(n * K, exp(eta_y))
 
-## ------------------------------------------------------------------------
-gm1 <- mixed_model(fixed = y ~ sex * time, random = ~ 1 | id, data = DF,
-                   family = poisson())
+## ---- eval = FALSE-------------------------------------------------------
+#  gm1 <- mixed_model(fixed = y ~ sex * time, random = ~ 1 | id, data = DF,
+#                     family = poisson())
 
-## ------------------------------------------------------------------------
-summary(gm1)
+## ---- eval = FALSE-------------------------------------------------------
+#  summary(gm1)
+#  #>
+#  #> Call:
+#  #> mixed_model(fixed = y ~ sex * time, random = ~1 | id, data = DF,
+#  #>     family = poisson())
+#  #>
+#  #> Data Descriptives:
+#  #> Number of Observations: 800
+#  #> Number of Groups: 100
+#  #>
+#  #> Model:
+#  #>  family: poisson
+#  #>  link: log
+#  #>
+#  #> Fit statistics:
+#  #>    log.Lik      AIC      BIC
+#  #>  -2782.331 5574.663 5587.689
+#  #>
+#  #> Random effects covariance matrix:
+#  #>                StdDev
+#  #> (Intercept) 0.9785518
+#  #>
+#  #> Fixed effects:
+#  #>                Estimate Std.Err  z-value   p-value
+#  #> (Intercept)      2.9461  0.3091   9.5316   < 1e-04
+#  #> sexfemale       -0.8651  0.2711  -3.1913 0.0014162
+#  #> time             0.2404  0.0016 151.3470   < 1e-04
+#  #> sexfemale:time  -0.0511  0.0028 -18.3994   < 1e-04
+#  #>
+#  #> Integration:
+#  #> method: adaptive Gauss-Hermite quadrature rule
+#  #> quadrature points: 11
+#  #>
+#  #> Optimization:
+#  #> method: hybrid EM and quasi-Newton
+#  #> converged: TRUE
 
-## ------------------------------------------------------------------------
-gm2 <- mixed_model(fixed = y ~ sex * time, random = ~ 1 | id, data = DF,
-                   family = poisson(), penalized = TRUE)
+## ---- eval = FALSE-------------------------------------------------------
+#  gm2 <- mixed_model(fixed = y ~ sex * time, random = ~ 1 | id, data = DF,
+#                     family = poisson(), penalized = TRUE)
 
-## ------------------------------------------------------------------------
-cbind('unpenalized' = fixef(gm1), 'penalized' = fixef(gm2))
+## ---- eval = FALSE-------------------------------------------------------
+#  cbind('unpenalized' = fixef(gm1), 'penalized' = fixef(gm2))
+#  #>                unpenalized   penalized
+#  #> (Intercept)     2.94614349  2.94238986
+#  #> sexfemale      -0.86510298 -0.82073289
+#  #> time            0.24039622  0.24043929
+#  #> sexfemale:time -0.05106264 -0.05118503
 
 ## ---- eval = FALSE-------------------------------------------------------
 #  gm3 <- mixed_model(fixed = y ~ sex * time, random = ~ 1 | id, data = DF,
